@@ -1,7 +1,7 @@
 /*
 	Description: custom action handling
 
-	Author: Marco Costalba (C) 2006
+	Author: Marco Costalba (C) 2006-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
@@ -9,9 +9,9 @@
 #ifndef CUSTOMACTIONIMPL_H
 #define CUSTOMACTIONIMPL_H
 
-#include "customactionbase.h"
+#include "ui_customaction.h"
 
-class CustomActionImpl : public CustomActionBase {
+class CustomActionImpl : public QWidget, public Ui_CustomActionBase {
 Q_OBJECT
 public:
 	CustomActionImpl();
@@ -20,7 +20,8 @@ signals:
 	void listChanged(const QStringList&);
 
 protected slots:
-	void listViewNames_currentChanged(QListViewItem*);
+	virtual void closeEvent(QCloseEvent*);
+	void listWidgetNames_currentItemChanged(QListWidgetItem*, QListWidgetItem*);
 	void pushButtonNew_clicked();
 	void pushButtonRename_clicked();
 	void pushButtonRemove_clicked();
@@ -32,13 +33,11 @@ protected slots:
 	void pushButtonOk_clicked();
 
 private:
-	void updateActionList();
+	const QStringList actions();
+	void updateActions();
 	bool getNewName(QString& name, const QString& caption);
 	void loadAction(const QString& name);
 	void removeAction(const QString& name);
-
-	QStringList actionList;
-	QString curAction;
 };
 
 #endif
