@@ -1,5 +1,5 @@
 /*
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
@@ -7,66 +7,53 @@
 #ifndef DATALOADER_H
 #define DATALOADER_H
 
-#include <qobject.h>
-#include <qtimer.h>
-#include <qdatetime.h>
-#include <qfile.h>
+#include <QProcess>
+#include <QTime>
+#include <QTimer>
 
-class QString;
 class Git;
 class FileHistory;
-class QProcess;
-class MyProcess;
+class QString;
+class UnbufferedTemporaryFile;
 
-// data exchange facility with git-rev-list could be based on QProcess or on
+// data exchange facility with 'git log' could be based on QProcess or on
 // a temporary file (default). Uncomment following line to use QProcess
 // #define USE_QPROCESS
 
-class DataLoader : public QObject {
+class DataLoader : public QProcess {
 Q_OBJECT
 public:
 	DataLoader(Git* g, FileHistory* f);
 	~DataLoader();
-	bool start(const QStringList& args, const QString& wd);
-
-public slots:
-	void on_procDataReady(const QByteArray&);
-	void on_eof();
+	bool start(const QStringList& args, const QString& wd, const QString& buf);
 
 signals:
 	void newDataReady(const FileHistory*);
-	void loaded(const FileHistory*,ulong,int,bool,const QString&,const QString&);
+	void loaded(FileHistory*,ulong,int,bool,const QString&,const QString&);
 
 private slots:
+	void on_finished(int, QProcess::ExitStatus);
 	void on_cancel();
 	void on_cancel(const FileHistory*);
 	void on_timeout();
 
 private:
 	void parseSingleBuffer(const QByteArray& ba);
+	void baAppend(QByteArray** src, const char* ascii, int len);
 	void addSplittedChunks(const QByteArray* halfChunk);
-	bool doStart(const QStringList& args, const QString& wd);
+	bool createTemporaryFile();
 	ulong readNewData(bool lastBuffer);
 
 	Git* git;
 	FileHistory* fh;
 	QByteArray* halfChunk;
+	UnbufferedTemporaryFile* dataFile;
 	QTime loadTime;
 	QTimer guiUpdateTimer;
 	ulong loadedBytes;
 	bool isProcExited;
 	bool parsing;
 	bool canceling;
-
-#ifdef USE_QPROCESS
-	QProcess* proc;
-#else
-	MyProcess* proc;
-	QString procPID;
-	QString scriptFileName;
-	QString dataFileName;
-	QFile dataFile;
-#endif
 };
 
 #endif
