@@ -20,6 +20,7 @@ class ListViewProxy;
 
 class ListView: public QTreeView {
 Q_OBJECT
+	struct DropInfo;
 public:
 	ListView(QWidget* parent);
 	~ListView();
@@ -28,6 +29,7 @@ public:
 	void showIdValues();
 	void scrollToCurrent(ScrollHint hint = EnsureVisible);
 	void scrollToNextHighlighted(int direction);
+	void scrollToNext(int direction);
 	void getSelectedItems(QStringList& selectedItems);
 	bool update();
 	void addNewRevs(const QVector<QString>& shaVec);
@@ -41,11 +43,14 @@ public:
 
 signals:
 	void lanesContextMenuRequested(const QStringList&, const QStringList&);
-	void revisionsDragged(const QStringList&);
-	void revisionsDropped(const QStringList&);
+	void applyRevisions(const QStringList& shas, const QString& remoteRepo);
+	void applyPatches(const QStringList &files);
+	void rebase(const QString& from, const QString& to, const QString& onto);
+	void merge(const QStringList& shas, const QString& into);
+	void moveRef(const QString& refName, const QString& toSHA);
 	void contextMenu(const QString&, int);
 	void diffTargetChanged(int); // used by new model_view integration
-	void showStatusMessage(const QString&);
+	void showStatusMessage(const QString&, int timeout=0);
 
 public slots:
 	void on_changeFont(const QFont& f);
@@ -58,7 +63,10 @@ protected:
 	virtual void mouseReleaseEvent(QMouseEvent* e);
 	virtual void dragEnterEvent(QDragEnterEvent* e);
 	virtual void dragMoveEvent(QDragMoveEvent* e);
+	virtual void dragLeaveEvent(QDragLeaveEvent* event);
 	virtual void dropEvent(QDropEvent* e);
+	void startDragging(QMouseEvent *e);
+	QPixmap pixmapFromSelection(const QStringList &revs, const QString &ref) const;
 
 private slots:
 	void on_customContextMenuRequested(const QPoint&);
@@ -78,9 +86,11 @@ private:
 	unsigned long secs;
 	bool filterNextContextMenuRequest;
 	QString lastRefName; // last ref name clicked on
+	DropInfo *dropInfo; // struct describing to-be-dropped content
 };
 
 class ListViewDelegate : public QItemDelegate {
+	friend class ListView;
 Q_OBJECT
 public:
 	ListViewDelegate(Git* git, ListViewProxy* lp, QObject* parent);
@@ -104,6 +114,7 @@ private:
 	QPixmap* getTagMarks(SCRef sha, const QStyleOptionViewItem& opt) const;
 	void addTextPixmap(QPixmap** pp, SCRef txt, const QStyleOptionViewItem& opt) const;
 	bool changedFiles(SCRef sha) const;
+	qreal dpr(void) const;
 
 	Git* git;
 	ListViewProxy* lp;
