@@ -1,5 +1,5 @@
 /*
-	Author: Marco Costalba (C) 2005-2006
+	Author: Marco Costalba (C) 2005-2007
 
 	Copyright: See COPYING file that comes with this distribution
 
@@ -7,33 +7,34 @@
 #ifndef REVSVIEW_H
 #define REVSVIEW_H
 
-#include <qguardedptr.h>
-#include "revbase.h" // needed by moc_* file to understand tab() function
+#include <QPointer>
+#include "ui_revsview.h" // needed by moc_* file to understand tab() function
 #include "common.h"
 #include "domain.h"
 
 class MainImpl;
 class Git;
 class FileHistory;
-class ListView;
-class ListBoxFiles;
 class PatchView;
-class TreeView;
 
 class RevsView : public Domain {
 Q_OBJECT
 public:
-	RevsView(MainImpl* parent, Git* git);
+	RevsView(MainImpl* parent, Git* git, bool isMain = false);
 	~RevsView();
-	void clear(bool keepState);
+	void clear(bool complete);
 	void viewPatch(bool newTab);
 	void setEnabled(bool b);
-	virtual TabRev* tab() { return revTab; }
+	void setTabLogDiffVisible(bool);
+	Ui_TabRev* tab() { return revTab; }
+
+public slots:
+	void toggleDiffView();
 
 private slots:
+	void on_newRevsAdded(const FileHistory*, const QVector<ShaString>&);
 	void on_loadCompleted(const FileHistory*, const QString& stats);
 	void on_lanesContextMenuRequested(const QStringList&, const QStringList&);
-	void on_droppedRevisions(const QStringList&);
 	void on_updateRevDesc();
 
 protected:
@@ -42,15 +43,10 @@ protected:
 private:
 	friend class MainImpl;
 
-	bool listBoxFiles_mouseMoveEvent();
-	bool listBoxFiles_mouseReleaseEvent();
 	void updateLineEditSHA(bool clear = false);
 
-	TabRev* revTab;
-	QGuardedPtr<PatchView> linkedPatchView;
-	TreeView* treeView;
-	ListView* listViewLog;
-	ListBoxFiles* listBoxFiles;
+	Ui_TabRev* revTab;
+	QPointer<PatchView> linkedPatchView;
 };
 
 #endif
